# Generated by roxygen2: do not edit by hand

export(autoGPoMoSearch)
export(autoGPoMoTest)
export(bDrvFilt)
export(cano2M)
export(combiEq)
export(compDeriv)
export(concat)
export(concatMulTS)
export(d2pMax)
export(derivODEwMultiX)
export(drvSucc)
export(findAllSets)
export(gPoMo)
export(gloMoId)
export(numiMultiX)
export(numicano)
export(numinoisy)
export(p2dMax)
export(pTimEv)
export(poLabs)
export(predictab)
export(regOrd)
export(regSeries)
export(subSysD)
export(testP)
export(visuEq)
export(visuOutGP)
export(wInProd)
import(deSolve)
import(float)
import(rgl)
importFrom(grDevices,dev.new)
importFrom(graphics,lines)
importFrom(graphics,par)
importFrom(graphics,plot)
importFrom(graphics,text)
importFrom(stats,lsfit)
importFrom(stats,qt)
importFrom(stats,quantile)
importFrom(stats,rnorm)
importFrom(stats,sd)
importFrom(stats,spline)
importFrom(utils,tail)
